@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2007-2008  Red Hat, Inc.
+# Copyright 2007-2011  Red Hat, Inc.
 # This file is part of python-fedora
 # 
 # python-fedora is free software; you can redistribute it and/or
@@ -30,7 +30,7 @@ from textwrap import wrap
 from os.path import join, expanduser, exists
 
 from fedora.client import BaseClient, FedoraClientError
-from fedora import _
+from fedora import b_
 
 __version__ = '0.5.1'
 log = logging.getLogger(__name__)
@@ -158,18 +158,20 @@ class BodhiClient(BaseClient):
                 'action': request,
                 })
 
-    def comment(self, update, comment, karma=0):
+    def comment(self, update, comment, karma=0, email=True):
         """ Add a comment to an update.
 
         :arg update: The title of the update comment on.
         :arg comment: The text of the comment.
         :kwarg karma: The karma of this comment (-1, 0, 1)
+        :kwarg email: Whether or not to trigger email notifications
 
         """
         return self.send_request('comment', auth=True, req_params={
                 'karma': karma,
                 'title': update,
                 'text': comment,
+                'email': email,
                 })
 
     def delete(self, update):
@@ -188,7 +190,7 @@ class BodhiClient(BaseClient):
         could potentially be pushed as updates.
         """
         if not self.username:
-            raise BodhiClientException(_('You must specify a username'))
+            raise BodhiClientException(b_('You must specify a username'))
         data = self.send_request('candidate_tags')
         koji = self.get_koji_session(login=False)
         for tag in data['tags']:
@@ -258,7 +260,7 @@ class BodhiClient(BaseClient):
 
         """
         from iniparse.compat import ConfigParser
-        self.log.info(_('Reading from %s ') % input_file)
+        self.log.info(b_('Reading from %s ') % input_file)
         input_file = expanduser(input_file)
         if exists(input_file):
             defaults = {
@@ -385,3 +387,15 @@ class BodhiClient(BaseClient):
         return session
 
     koji_session = property(fget=get_koji_session)
+
+    def get_releases(self):
+        """ Return a list of bodhi releases.
+
+        This method returns a dictionary in the following format::
+
+            {"releases": [
+                {"dist_tag": "dist-f12", "id_prefix": "FEDORA",
+                 "locked": false, "name": "F12", "long_name": "Fedora 12"}]}
+        """
+        return self.send_request('releases')
+
